% Tune and train supervised Machine Learning : KNN, tree, ensemble and SVM
% classifier
% Before running, check input/output directory (in all of the readtable/writetable functions)

clear;
addpath(genpath('D:\Maeva\Data MC Maeva\Matlab tools'))
addpath(genpath('D:\Maeva\2020-2021\Scripts_nursings'))


niter = 30; %number of iterations
speed_th = 2; %speed threshold (data segmentation)
iter = 100;%hyperparam runs
jjiter=1; %Not used. Keep at 1.

da1 = table; da2 = table;
knn1 = table; knn2 = table;
tree1 = table; tree2 = table;
rf1 = table; rf2 = table;
svm1 = table; svm2 = table;

for i=2:2 %specify here the windows size (2:2 = process 2s windows data)
output2 = ['Supplemental_Data_S2']; %csv filename
%%
% Import the data
nursingsfeaturesnew = readtable(['D:\Maeva\2020-2021\CATS table\Suckl\' output2 '.csv']);
nursingsfeaturesnew.FSRmean(isnan(nursingsfeaturesnew.FSRmean))=0; % Replace NaNs in FSRmean with zero (gliding)

% %%%%%%%%%%%%%%%%%%%%%%%%
depid(1) = {'Jaune_14sept2018_calf_new'}; 
depid(2) = {'Jaune_06aout2019_calf_new'}; 
depid(3) = {'Orange_09aout2019_calf_new'}; 
nursingsfeaturesnew.traveling(nursingsfeaturesnew.smean > speed_th,:)=1; %find high speed blocks
Calf1 = nursingsfeaturesnew(strcmp(nursingsfeaturesnew.nc_file,char(depid(1))),:); Calf1 = sortrows(Calf1);Calf1 = filtertravel(Calf1);
Calf2 = nursingsfeaturesnew(strcmp(nursingsfeaturesnew.nc_file,char(depid(2))),:); Calf2 = sortrows(Calf2);Calf2 = filtertravel(Calf2);
Calf3 = nursingsfeaturesnew(strcmp(nursingsfeaturesnew.nc_file,char(depid(3))),:); Calf3 = sortrows(Calf3);Calf3 = filtertravel(Calf3);

nursingsfeaturesnew = [Calf1;Calf2;Calf3];
nursingsfeaturesnew = nursingsfeaturesnew(nursingsfeaturesnew.traveling2 == 0,:); tabulate(nursingsfeaturesnew.clust)

nursingsfeaturesnew = nursingsfeaturesnew(nursingsfeaturesnew.Pmean > 1.5,:);tabulate(nursingsfeaturesnew.clust) %keep only blocks at >1.5 m depth

%%

for ii = 1:niter
cvp = cvpartition(string(nursingsfeaturesnew.clust) + ' - ' + string(nursingsfeaturesnew.nc_file), 'Holdout', 0.4); % Partition the data
testset = nursingsfeaturesnew(cvp.test, :); %Test set
trainset = nursingsfeaturesnew(cvp.training, :); %Train set

for jj=1:jjiter

 nn = (ii*jjiter)-(jjiter-jj);

% -------------------------------------------------------------------------
% KNN optimization
% -------------------------------------------------------------------------
[trainedknn, Accuracyknn, knnresults] = knnClassifierOpt(trainset,iter); % Run Optimization
trainedknn.Tuning = knnresults;
trainedknn.accuracy = Accuracyknn;
knnopt.model(nn)= trainedknn; % Windows
 
knnpred = trainedknn.predictFcn(testset);
stats = confusionmatStats(char(testset.clust),char(knnpred));%http://www.mathworks.com/matlabcentral/fileexchange/46035-confusion-matrix--accuracy--precision--specificity--sensitivity--recall--f-score
knn1.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
knn1.clust(nn) = {'Non Nursing'};
knn1.sensitivity(nn) = stats.sensitivity(1);
knn1.FP(nn) = 1-stats.specificity(1);
knn1.precision(nn) = stats.precision(1);
knn1.specificity(nn) = stats.specificity(1);
knn1.Fscore(nn) = stats.Fscore(1);
knn1.global_accuracy(nn) = stats.accuracy;
knn1.windows(nn)= {[num2str(i) 's']}; % Windows
knn1.method(nn)= {'knn'}; % Method
knn1.type(nn)= {trainedknn.ClassificationKNN.Distance}; %type
knn2.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
knn2.clust(nn) = {'Nursing'};
knn2.sensitivity(nn) = stats.sensitivity(2);
knn2.FP(nn) = 1-stats.specificity(2);
knn2.precision(nn) = stats.precision(2);
knn2.specificity(nn) = stats.specificity(2);
knn2.Fscore(nn) = stats.Fscore(2);
knn2.global_accuracy(nn) = stats.accuracy;
knn2.windows(nn)= {[num2str(i) 's']}; % Windows
knn2.method(nn)= {'knn'}; % Method
knn2.type(nn)= {trainedknn.ClassificationKNN.Distance}; %type
disp([knn1;knn2])
 disp(['knn : ITERATION No' num2str(ii) ' COMPLETED'])
 close all;
 
% -------------------------------------------------------------------------
% tree optimization
% -------------------------------------------------------------------------
[trainedtree, Accuracytree, treeresults] = treeClassifierOpt(trainset,iter); % Run Optimization
trainedtree.Tuning = treeresults;
trainedtree.accuracy = Accuracytree;
treeopt.model(nn)= trainedtree; % Windows
 
treepred = trainedtree.predictFcn(testset);
stats = confusionmatStats(char(testset.clust),char(treepred));%http://www.mathworks.com/matlabcentral/fileexchange/46035-confusion-matrix--accuracy--precision--specificity--sensitivity--recall--f-score
tree1.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
tree1.clust(nn) = {'Non Nursing'};
tree1.sensitivity(nn) = stats.sensitivity(1);
tree1.FP(nn) = 1-stats.specificity(1);
tree1.precision(nn) = stats.precision(1);
tree1.specificity(nn) = stats.specificity(1);
tree1.Fscore(nn) = stats.Fscore(1);
tree1.global_accuracy(nn) = stats.accuracy;
tree1.windows(nn)= {[num2str(i) 's']}; % Windows
tree1.method(nn)= {'tree'}; % Method
tree1.type(nn)= {treeresults.XAtMinEstimatedObjective.tree_SplitCriterion}; %type
tree2.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
tree2.clust(nn) = {'Nursing'};
tree2.sensitivity(nn) = stats.sensitivity(2);
tree2.FP(nn) = 1-stats.specificity(2);
tree2.precision(nn) = stats.precision(2);
tree2.specificity(nn) = stats.specificity(2);
tree2.Fscore(nn) = stats.Fscore(2);
tree2.global_accuracy(nn) = stats.accuracy;
tree2.windows(nn)= {[num2str(i) 's']}; % Windows
tree2.method(nn)= {'tree'}; % Method
tree2.type(nn)= {treeresults.XAtMinEstimatedObjective.tree_SplitCriterion}; %type
disp([tree1;tree2])
 disp(['tree : ITERATION No' num2str(ii) ' COMPLETED'])
 close all;
 
% -------------------------------------------------------------------------
% Ensemble classifier optimization
% -------------------------------------------------------------------------
[trainedrf, Accuracyrf, rfresults] = rfClassifierOpt(trainset,iter); % Run Optimization
trainedrf.Tuning = rfresults;
trainedrf.accuracy = Accuracyrf;
rfopt.model(nn)= trainedrf; % Windows
 
rfpred = trainedrf.predictFcn(testset);
stats = confusionmatStats(char(testset.clust),char(rfpred));%http://www.mathworks.com/matlabcentral/fileexchange/46035-confusion-matrix--accuracy--precision--specificity--sensitivity--recall--f-score
rf1.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
rf1.clust(nn) = {'Non Nursing'};
rf1.sensitivity(nn) = stats.sensitivity(1);
rf1.FP(nn) = 1-stats.specificity(1);
rf1.precision(nn) = stats.precision(1);
rf1.specificity(nn) = stats.specificity(1);
rf1.Fscore(nn) = stats.Fscore(1);
rf1.global_accuracy(nn) = stats.accuracy;
rf1.windows(nn)= {[num2str(i) 's']}; % Windows
rf1.method(nn)= {'ensemble'}; % Method
rf1.type(nn)= {rfresults.XAtMinEstimatedObjective.ensemble_Method}; %type
rf2.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
rf2.clust(nn) = {'Nursing'};
rf2.sensitivity(nn) = stats.sensitivity(2);
rf2.FP(nn) = 1-stats.specificity(2);
rf2.precision(nn) = stats.precision(2);
rf2.specificity(nn) = stats.specificity(2);
rf2.Fscore(nn) = stats.Fscore(2);
rf2.global_accuracy(nn) = stats.accuracy;
rf2.windows(nn)= {[num2str(i) 's']}; % Windows
rf2.method(nn)= {'ensemble'}; % Method
rf2.type(nn)= {rfresults.XAtMinEstimatedObjective.ensemble_Method}; %type
disp([rf1;rf2])
 disp(['ensemble : ITERATION No' num2str(ii) ' COMPLETED'])
 close all;
 
% -------------------------------------------------------------------------
% SVM optimization
% -------------------------------------------------------------------------
[trainedsvm, Accuracysvm, svmresults] = svmClassifierOpt(trainset,iter); % Run Optimization
trainedsvm.Tuning = svmresults;
trainedsvm.accuracy = Accuracysvm;
svmopt.model(nn)= trainedsvm; % Windows
 
svmpred = trainedsvm.predictFcn(testset);
stats = confusionmatStats(char(testset.clust),char(svmpred));%http://www.mathworks.com/matlabcentral/fileexchange/46035-confusion-matrix--accuracy--precision--specificity--sensitivity--recall--f-score
svm1.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
svm1.clust(nn) = {'Non Nursing'};
svm1.sensitivity(nn) = stats.sensitivity(1);
svm1.FP(nn) = 1-stats.specificity(1);
svm1.precision(nn) = stats.precision(1);
svm1.specificity(nn) = stats.specificity(1);
svm1.Fscore(nn) = stats.Fscore(1);
svm1.global_accuracy(nn) = stats.accuracy;
svm1.windows(nn)= {[num2str(i) 's']}; % Windows
svm1.method(nn)= {'svm'}; % Method
svm1.type(nn)= {svmresults.XAtMinEstimatedObjective.svm_KernelFunction}; %type
svm2.iter(nn) = {[num2str(ii) '-' num2str(jj)]};
svm2.clust(nn) = {'Nursing'};
svm2.sensitivity(nn) = stats.sensitivity(2);
svm2.FP(nn) = 1-stats.specificity(2);
svm2.precision(nn) = stats.precision(2);
svm2.specificity(nn) = stats.specificity(2);
svm2.Fscore(nn) = stats.Fscore(2);
svm2.global_accuracy(nn) = stats.accuracy;
svm2.windows(nn)= {[num2str(i) 's']}; % Windows
svm2.method(nn)= {'svm'}; % Method
svm2.type(nn)= {svmresults.XAtMinEstimatedObjective.svm_KernelFunction}; %type
disp([svm1;svm2])
 disp(['svm : ITERATION No' num2str(ii) ' COMPLETED'])
 
dapred = NaN;
save(['D:\Maeva\2020-2021\Scripts_nursings\Mat\partition' num2str(ii) '-' num2str(jj) '.mat'], 'testset','trainset', 'dapred','knnpred','treepred','rfpred', 'svmpred')

end
end

end

final_table = [da1;da2;knn1;knn2;tree1;tree2;rf1;rf2;svm1;svm2];
disp(final_table)

%
daopt = NaN;
save('D:\Maeva\2020-2021\Scripts_nursings\Mat\results_all.mat','final_table','daopt','knnopt','treeopt','rfopt', 'svmopt')
%
writetable(final_table,['D:\Maeva\2020-2021\Scripts_nursings\Mat\ML_all' num2str(i) 's.csv'],'Delimiter',',');